import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:hqsoft_core/core_base/core_base.dart';
import 'package:hqsoft_core/value_base/value_base.dart';

class BoxFieldPhone extends StatefulWidget {
  final TextEditingController? controller;
  final FocusNode? focusNode;
  final TextInputType? keyboardType;
  final TextInputAction? textInputAction;
  final TextCapitalization? textCapitalization;
  final TextStyle? style;
  final TextStyle? hintStyle;
  final TextAlign? textAlign;
  final bool isRequired;
  final bool autoFocus;
  final bool obscureText;
  final bool autoCorrect;
  final int maxLines;
  final Key? key;
  final int? maxLength;
  final bool maxLengthEnforced;
  final ValueChanged<String>? onChanged;
  final VoidCallback? onEditingComplete;
  final ValueChanged<String>? onSubmitted;
  final List<TextInputFormatter>? inputFormatter;
  final bool? enabled;
  final IconData? icon;
  final Color? iconColor;
  final Color? focusBorderColor;
  final String? hintText;
  final double? cursorWidth;
  final Radius? cursorRadius;
  final Color? cursorColor;
  final Color? defaultBorderColor;
  final Brightness? keyboardAppearance;
  final EdgeInsets? scrollPadding;
  final double? heightCont;
  final FormFieldValidator<String>? validator;
  final ValueChanged<String>? onFieldSubmitted;

  final FormFieldSetter<String>? onSaved;

  const BoxFieldPhone({
    this.key,
    this.controller,
    this.heightCont,
    this.focusNode,
    this.keyboardType,
    this.textInputAction,
    this.textCapitalization,
    this.style,
    this.hintStyle,
    this.icon,
    this.textAlign = TextAlign.start,
    this.isRequired = false,
    this.autoFocus = false,
    this.obscureText = false,
    this.autoCorrect = true,
    this.maxLines = 1,
    this.maxLength,
    this.onSaved,
    this.hintText,
    this.maxLengthEnforced = true,
    this.onChanged,
    this.defaultBorderColor,
    this.onEditingComplete,
    this.onSubmitted,
    this.iconColor,
    this.inputFormatter,
    this.enabled,
    this.cursorWidth = 2.0,
    this.cursorRadius,
    this.cursorColor,
    this.focusBorderColor,
    this.keyboardAppearance,
    this.scrollPadding,
    this.validator,
    this.onFieldSubmitted,
  })  : assert(textAlign != null),
        assert(isRequired != null),
        assert(autoFocus != null),
        assert(obscureText != null),
        assert(autoCorrect != null),
        assert(maxLengthEnforced != null),
        assert(maxLines == null || maxLines > 0),
        assert(maxLength == null || maxLength > 0);

//        keyboardType = keyboardType ??
//            (maxLines == 1 ? TextInputType.text : TextInputType.multiline);

  @override
  _BoxFieldPhoneState createState() => _BoxFieldPhoneState();
}

class _BoxFieldPhoneState extends State<BoxFieldPhone> {
  double? width;
  double? height;
  double? heightCont;
  double? iconSize;
  Color focusBorderColor = Colors.grey.shade400;
  FocusNode _focusNode = FocusNode();
  ValueChanged<Colors>? focusColorChange;

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
    _focusNode.dispose();
  }

  @override
  Widget build(BuildContext context) {
    width = MediaQuery.of(context).size.width;
    height = MediaQuery.of(context).size.height;
    heightCont = widget.heightCont == null ? 45 : widget.heightCont;
    iconSize = widget.heightCont == null ? 24 : 18;

    return Container(
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Expanded(
            child: Container(
              padding: EdgeInsets.all(ScreenConfig.getWidthPx(0)),
              alignment: Alignment.center,
              height: ScreenConfig.getWidthPx(heightCont!.toInt()),
              decoration: BoxDecoration(
                color: Colors.grey.shade100,
                border: Border.all(
                    color: widget.focusBorderColor ?? Colors.grey.shade400,
                    width: 1.0),
                borderRadius: BorderRadius.only(
                  topRight: Radius.circular(8.0),
                  bottomRight: Radius.circular(8.0),
                ),
              ),
              child: TextFormField(
                key: widget.key,
                style: ThemeBase.textBoxFieldBlack,
                obscureText: widget.obscureText,
                controller: widget.controller,
                onSaved: widget.onSaved,
                validator: widget.validator,
                focusNode: widget.focusNode,
                onFieldSubmitted: widget.onFieldSubmitted,
                textInputAction: widget.textInputAction,
                textCapitalization: widget.textCapitalization ?? TextCapitalization.none,
                keyboardType: widget.keyboardType == null
                    ? (widget.maxLines == 1
                        ? TextInputType.text
                        : TextInputType.multiline)
                    : widget.keyboardType,
                inputFormatters: widget.inputFormatter,
                decoration: InputDecoration(
                    border: InputBorder.none,
                    prefixIcon: Icon(
                      widget.icon,
                      color: widget.iconColor,
                      size: ScreenConfig.getWidthPx(iconSize!.toInt()),
                    ),
                    hintStyle: ThemeBase.textBoxFieldHint,
                    hintText: widget.hintText),
              ),
            ),
          ),
          widget.isRequired
              ? Text(" * ", style: ThemeBase.textHaveRequired)
              : Text("    ", style: ThemeBase.textNonRequired),
        ],
      ),
      padding: EdgeInsets.only(bottom: ScreenConfig.getWidthPx(0)),
      margin: EdgeInsets.only(
          top: ScreenConfig.getWidthPx(8), right: ScreenConfig.getWidthPx(0)),
    );
  }
}
